@@ -7,6 +7,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
@@ -59,14 +60,6 @@ static struct decryption_key *aes_key = NULL;
 
 static char *fwenv_config = NULL;
 
-char *sdup(const char *str) {
-	char *p;
-	if ((p = (char *) malloc(strlen(str) + 1)) != NULL) {
-		strcpy(p, str);
-	}
-	return p;
-}
-
 static char* TMPDIR = NULL;
 static char* TMPDIRSCRIPT = NULL;
 
@@ -169,6 +162,26 @@ void swupdate_create_directory(const char* path) {
 }
 
 #ifndef CONFIG_NOCLEANUP
+static int _is_mount_point(const char *path, const char *parent_path) {
+	struct stat path_stat, parent_stat;
+
+	if (stat(path, &path_stat)) {
+		ERROR("stat for path %s failed: %s", path, strerror(errno));
+		return -errno;
+	}
+
+	if (stat(parent_path, &parent_stat)) {
+		ERROR("stat for parent path %s failed: %s", parent_path, strerror(errno));
+		return -errno;
+	}
+
+	if (path_stat.st_dev != parent_stat.st_dev) {
+		return 1;
+	}
+
+	return 0;
+}
+
 static int _remove_directory_cb(const char *fpath, const struct stat *sb,
 								int typeflag, struct FTW *ftwbuf)
 {
@@ -187,7 +200,23 @@ int swupdate_remove_directory(const char* path)
 		ERROR("OOM: Directory %s not removed", path);
 		return -ENOMEM;
 	}
+
+	ret = _is_mount_point(dpath, get_tmpdir());
+	if (ret < 0)
+		goto out;
+
+	if (ret) {
+		WARN("Unexpected mountpoint, unmounting: %s", dpath);
+		ret = swupdate_umount(dpath);
+		if (ret && errno != EINVAL) {
+			ret = -errno;
+			ERROR("Can't unmount path %s: %s", dpath, strerror(errno));
+			goto out;
+		}
+	}
+
 	ret = nftw(dpath, _remove_directory_cb, 64, FTW_DEPTH | FTW_PHYS);
+out:
 	free(dpath);
 	return ret;
 }
@@ -267,6 +296,23 @@ char *mstrcat(const char **nodes, const char *delim)
 	return dest;
 }
 
+char *swupdate_strcat(int n, ...)
+{
+	const char *nodes[n + 1];
+
+	va_list valist;
+
+	va_start(valist, n);
+
+	for(int i = 0; i < n; i++)
+		nodes[i] = va_arg(valist, const char *);
+	nodes[n] = NULL;
+
+	va_end(valist);
+
+	return mstrcat(nodes, NULL);
+}
+
 /*
  * Alocate and return a string as part of
  * another string
@@ -816,6 +862,69 @@ int swupdate_umount(const char *dir)
 #endif
 }
 
+static const char *mount_points[] = {
+	[MNT_SCRIPTS] = SCRIPTS_DIR_SUFFIX,
+	[MNT_DATA] = DATADST_DIR_SUFFIX,
+	[MNT_BOOT_SCRIPTS] = BOOT_SCRIPT_SUFFIX
+};
+
+char *swupdate_temporary_mount(tmp_mountpoint_t type, const char *device, const char *fstype)
+{
+	char *mountpoint;
+	const char *dir;
+	int ret = 0;
+
+	if (type != MNT_SCRIPTS && type != MNT_DATA && type != MNT_BOOT_SCRIPTS)
+		return NULL;
+
+	if (!device || !strlen(device)) {
+		ERROR("Mount requested without Device");
+		return NULL;
+	}
+
+	dir = mount_points[type];
+	if (asprintf(&mountpoint, "%s%sXXXXXX", get_tmpdir(), dir) == -1) {
+		ERROR("Unable to allocate memory");
+		return NULL;
+	}
+
+	if (!mkdtemp(mountpoint)) {
+		TRACE("Unable to create a unique temporary directory %s: %s",
+			mountpoint, strerror(errno));
+		free(mountpoint);
+		return NULL;
+	}
+
+	ret = swupdate_mount(device, mountpoint, fstype);
+	if (ret) {
+		TRACE("Device %s with filesystem %s cannot be mounted: %s",
+			device, fstype, strerror(errno));
+		rmdir(mountpoint);
+		free(mountpoint);
+		return NULL;
+	}
+
+	return mountpoint;
+}
+
+int swupdate_temporary_umount(char *mountpoint)
+{
+	int ret = 0;
+	if (swupdate_umount(mountpoint)) {
+		TRACE("Unable to unmount %s: %s", mountpoint, strerror(errno));
+		ret = -EFAULT;
+	}
+	if (ret == 0 && rmdir(mountpoint) == -1) {
+		if (errno != EROFS) {
+			TRACE("Unable to remove directory %s: %s", mountpoint, strerror(errno));
+			ret = -EROFS;
+		}
+	}
+	free(mountpoint);
+
+	return ret;
+}
+
 /*
  * Date time in SWUpdate
  * @return : date in ISO8601 (it must be freed by caller)
